/****************************************************************************
*
*               Laborator SMP - 80386EX Z3/EV Real Mode
*
*    Portions Copyright (c) 2010 Mihai Matei. All Rights Reserved.
*
*  ========================================================================
*
*    This file contains Original Code and/or Modifications of Original
*    Code as defined in and that are subject to the Apache License,
*    Version 2.0 (the "License"). You may not use this file
*    except in compliance with the License. BY USING THIS FILE YOU AGREE TO
*    ALL TERMS AND CONDITIONS OF THE LICENSE. A copy of the License is
*    provided with the Original Code and Modifications, and is also
*    available at http://www.apache.org/licenses/LICENSE-2.0
*
*    The Original Code and all software distributed under the License are
*    distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
*    EXPRESS OR IMPLIED, AND SYBASE AND ALL CONTRIBUTORS HEREBY DISCLAIM
*    ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR
*    NON-INFRINGEMENT. Please see the License for the specific language
*    governing rights and limitations under the License.
*
*  ========================================================================
*
* Description:  Common Interface for Platform specific service implementation
*
****************************************************************************/

#ifndef __PLATFORM_IO_H__
#define __PLATFORM_IO_H__


/********************************* Parallel interface types *********************************/
typedef enum {
    Mode0 = 0x0,
    Mode1 = 0x1,
    Mode2 = 0x2
} ParallelMode;

typedef enum {
    output = 0x0,
    input  = 0x1,
    inout  = 0x3
} ParallelDirection;

typedef enum {
    A     = 0x00,
    B     = 0x01,
    Clow  = 0x02,
    Chigh = 0x03
} ParallelPort;

typedef struct {
    unsigned char     hasInit;    /* If the interface has been initialized        */
    ParallelMode      opMode;     /* The operating mode of the Parallel interface */
    ParallelDirection portDir[4]; /* Data Direction of A, B, C(low), C(high)      */
} ParallelConfig, *pParallelConfig;
/********************************************************************************************/


/********************************* Serial Interface types ***********************************/
typedef enum {
    COM1 = 0x0,
    COM2 = 0x1,
    COM3 = 0x2,
} SerialPort;

typedef enum {
    b1200 = 0x0, /* '000b */
    b2400 = 0x1, /* '001b */
    b4800 = 0x2, /* '010b */
    b9600 = 0x3  /* '011b */
} BaudRate;

typedef enum {
    none = 0x0, /* '00b */
    odd  = 0x1, /* '01b */
    even = 0x3  /* '11b */
} Parity;

typedef enum {
    one = 0x0, /* 1 stop bit  */
    two = 0x1  /* 2 stop bits */
} StopBits;

typedef enum {
    seven = 0x2, /* '10b */
    eight = 0x3  /* '11b */
} WordLength;

typedef struct {
    unsigned char hasInit;    /* If the interface has been initialized      */
    BaudRate      baudRate;
    Parity        parity;
    StopBits      stopBits;
    WordLength    wordLength;
} SerialConfig, *pSerialConfig;
/********************************************************************************************/


/********************************** A/D | D/A Converter types *******************************/
#define BUFFER_SIZE 124
/* The Convert Data to be passed to the chips */
typedef struct {
    unsigned char size;                /* number of data chunks to input/output */
    unsigned char buffer[BUFFER_SIZE]; /* buffer to hold input/output data      */
} ConvertBuffer, *pConvertBuffer;
/********************************************************************************************/


/********************************* Interrupt Vector Types ***********************************/

/* An entry in the Interrupt Vector Table = an Interrupt Description */
typedef struct {
    unsigned char ip[2]; /* Interrupt Service Routine IP */
    unsigned char cs[2]; /* Interrupt Service Routine CS */
} InterruptVector, * __far pInterruptVector;

extern const unsigned int kIntNo;

/* To access interrupt use: gIVT[0] = int 0x0,  gIVT[7] = int 0x7 */
extern pInterruptVector gIVT;

/* Type to access an interrupt by its number */
typedef unsigned char InterruptNumber;

/* IMPORTANT: The __interrupt keyword means that all registers are saved before the
 * function begins execution and restored prior to returning from the interrupt.
 * The machine language return instruction for the function is changed to iret */
/* An interrupt service routine type */
typedef void __interrupt (* ISR) (void);
/********************************************************************************************/


/********************* Global API for working with the EV/Z3 board **************************/
typedef const struct {
        /* Display API */
        void (*DisplayString) (const char *);
        void (*DisplayBuffer) (const unsigned char *, const int);
        /* Keyboard API */
        int  (*ReadKey)       (void);
        /* Parallel Interface */
        void (*ParallelInit)  (const pParallelConfig);
        int  (*ParallelWrite) (ParallelPort, const unsigned char *, const int);
        int  (*ParallelRead)  (ParallelPort, unsigned char *, const int);
        /* Serial Interface */
        void (*SerialInit)    (const pSerialConfig);
        int  (*SerialWrite)   (SerialPort, const unsigned char *, const int);
        int  (*SerialRead)    (SerialPort, unsigned char *, const int);
        /* A/D | D/A Converter Interface */
        void (*ADconvert)     (pConvertBuffer);
        void (*DAconvert)     (pConvertBuffer);
        /* Utility */        
        void (*MemCpy)        (const unsigned char __far *, const int, unsigned char *);
        /* System API */
        void (*Exit)          (void);
        void (*Wait)          (const unsigned int);
        void (*Buzzer)        (const unsigned int, const unsigned int);
        /* Private DATA per Interface */
        void* __extension[];
} PlatformIo, *pPlatformIo;

extern PlatformIo kIo;
/********************************************************************************************/

#endif

