/****************************************************************************
*
*               Laborator SMP - 80386EX Z3/EV Real Mode
*
*    Portions Copyright (c) 2010 Mihai Matei. All Rights Reserved.
*
*  ========================================================================
*
*    This file contains Original Code and/or Modifications of Original
*    Code as defined in and that are subject to the Apache License,
*    Version 2.0 (the "License"). You may not use this file
*    except in compliance with the License. BY USING THIS FILE YOU AGREE TO
*    ALL TERMS AND CONDITIONS OF THE LICENSE. A copy of the License is
*    provided with the Original Code and Modifications, and is also
*    available at http://www.apache.org/licenses/LICENSE-2.0
*
*    The Original Code and all software distributed under the License are
*    distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
*    EXPRESS OR IMPLIED, AND SYBASE AND ALL CONTRIBUTORS HEREBY DISCLAIM
*    ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR
*    NON-INFRINGEMENT. Please see the License for the specific language
*    governing rights and limitations under the License.
*
*  ========================================================================
*
* Description:  80386EX Z3/EV Monitor Software Implementation of PlatformIo Interface
*
****************************************************************************/


#include "../platform_io.h"


/************************************************* Z3/EV Monitor's Interrupts *****************************************************/
#define IEND           0x07 /* End of a program */
#define IREADKEY       0x08 /* Reads a key */
#define IDISPLAYBYTE   0x09 /* Displays a byte on the LCD */
#define IDISPLAYCHAR   0x0A /* Displays a char on the LCD */
#define IDISPLAYSTRING 0x0B /* Displays a string on the LCD */
#define ICHANGECURSOR  0x0C /* Changes the position of the LCD's cursor */
#define IWAIT          0x0D /* Waits a number of miliseconds */
#define IADCONVERT     0x0E /* Reads the Analogic/Digital Converter */
#define IDACONVERT     0x0F /* Sends a byte to the Digital/Analogic Converter */
#define IBUZZER        0x10 /* Sets the buzzer */
#define IIOPARALLEL    0x12 /* Management of the Parallel Interface */
#define IIOSERIAL      0x14 /* Management of the Serial Interface */
/**********************************************************************************************************************************/


/************************************************* Z3/EV Platform Constants *******************************************************/
const int kLCDSize   = 15;
const int kKeyNo     = 19;
const int kParCWAddr = 0x0313;

const unsigned int kIntNo = 256;
/* In Real Mode the Interrupt Vector Table is at 0000:0000 */
pInterruptVector gIVT = 0x00000000;
/* IMPORTANT: on 16 bit systems a far pointer is a 32 bit address in the form of selector:offset
 * The selector represents the segment value. A plain 32 bit address can be computed like:
 * 32bit address = selector << 4 + offset. For example a 32 bit address 0xF_7FFF can be
 * represented in 16 bit far pointer format like 0xF000_7FFF = 0xF000 << 4 + 0x7FFF = 0xF_7FFF */
const unsigned char __near * const kUserRam[]     = { (unsigned char __near *)     0x0000, (unsigned char __near *)     0x7FFF };
const unsigned char __far  * const kUserEprom[]   = { (unsigned char __far  *) 0xF0000000, (unsigned char __far  *) 0xF0007FFF };
const unsigned char __far  * const kSystemEprom[] = { (unsigned char __far  *) 0xF0008000, (unsigned char __far  *) 0xF000FFFF };
/**********************************************************************************************************************************/


/************************************************* Z3/EV Platform Globals *********************************************************/
static ParallelConfig gParallelConf;
static SerialConfig   gSerialConf; 
/**********************************************************************************************************************************/


/************************************************* The 80386EX Z3/EV Interface Implementation *************************************/
static void z3DisplayString(const char *string);
static void z3DisplayBuffer(const unsigned char *buffer, const int size);
static int  z3ReadKey(void);
static void z3MemCpy(const unsigned char __far *source, const int size, unsigned char *dest);
static void z3Exit(void);
static void z3Wait(const unsigned int miliSec);
static void z3Buzzer(const unsigned int duration, const unsigned int frequency);
static void z3ParallelInit(const pParallelConfig pConfig);
static int  z3ParallelWrite(ParallelPort port, const unsigned char *buffer, const int size);
static int  z3ParallelRead(ParallelPort port, unsigned char *buffer, const int size);
static void z3SerialInit(const pSerialConfig pConfig);
static int  z3SerialWrite(SerialPort port, const unsigned char *buffer, const int size);
static int  z3SerialRead(SerialPort port, unsigned char *buffer, const int size);
static void z3ADconvert(pConvertBuffer convertBuff);
static void z3DAconvert(pConvertBuffer convertBuff);

PlatformIo kIo = {
    .DisplayString = z3DisplayString,
    .DisplayBuffer = z3DisplayBuffer,
    .ReadKey       = z3ReadKey,
    .ParallelInit  = z3ParallelInit,
    .ParallelWrite = z3ParallelWrite,
    .ParallelRead  = z3ParallelRead,
    .SerialInit    = z3SerialInit,
    .SerialWrite   = z3SerialWrite,
    .SerialRead    = z3SerialRead,
    .ADconvert     = z3ADconvert,
    .DAconvert     = z3DAconvert,
    .MemCpy        = z3MemCpy,
    .Exit          = z3Exit,
    .Wait          = z3Wait,
    .Buzzer        = z3Buzzer,
    .__extension   = { &gParallelConf, &gSerialConf, 0 }
};
/**********************************************************************************************************************************/


static void z3DisplayString(const char *string)
{
    __asm {
        push SI
        mov  SI, string     ; Load the param into SI
        int  IDISPLAYSTRING ; Monitor Interrupt to print the string from DS:SI
        pop  SI
    }
}


static void z3DisplayBuffer(const unsigned char *buffer, const int size)
{
    int index   = 0;
    /* 2 chars are used to display one byte in hexa */
    int maxSize = (size > (kLCDSize >> 1))? (kLCDSize >> 1) : size;

    for (; index < maxSize; index ++) {
        unsigned char b = buffer[index];        
        __asm {
            push CX
            push AX

            xor  CX, CX       ; Clear CX just to be on the safe side
            xor  AX, AX       ; Clear AX just to be on the safe side
            mov  CX, index
            shl  CL, 1        ; CL = index * 2
            mov  AL, b        ; AL = buffer[index]
            int  IDISPLAYBYTE ; Displays a byte in AL at cursor position CL

            pop  AX
            pop  CX
        }
    }
}


static int  z3ReadKey(void)
{
    int key = 0;
    __asm {
        push AX
        xor  AX, AX   ; Clear AX just to be on the safe side
        int  IREADKEY ; Monitor Interrupt to wait for a key to be pressed
        mov  key, AX
        pop  AX
    }
    key = key % kKeyNo; /* Key Code must be 0-18 */
    return key;
}


static void z3MemCpy(const unsigned char __far *source, const int size, unsigned char *dest)
{
    int i = 0;
    int maxSize = size;

    /* check the correct ranges */
    if ((dest > kUserRam[1]) || (source < kUserEprom[0]) || (source > kSystemEprom[1]))
        return;    
    maxSize = ((dest + size)   < kUserRam[1])?        size : (int)(kUserRam[1] - dest);
    maxSize = ((source + size) < kSystemEprom[1])? maxSize : (int)(kSystemEprom[1] - source);

    for (; i < maxSize; i++) {
        dest[i] = source[i];
    }
}


static void z3Exit(void)
{
    __asm {
        int IEND ; Exit to Monitor Software Interrupt
    }
}


static void z3Wait(const unsigned int miliSec)
{
    __asm {
        push AX          ; Save AX
        mov  AX, miliSec
        int  IWAIT       ; Wait the number of miliseconds in AX
        pop  AX
    }
}


static void z3Buzzer(const unsigned int duration, const unsigned int frequency)
{
    __asm {
        push BX
        push CX

        mov BX, duration
        mov CX, frequency
        int IBUZZER ; beep with settup duration and frequency

        pop  BX
        pop  CX
    }
}


static void z3ParallelInit(const pParallelConfig pConfig)
{
    unsigned char cw = 1 << 7; /* Mod Set Flag = active */

    /* setup the operating mode by setting required bits */
    switch (pConfig->opMode) {
        case Mode0: break;
        case Mode1: cw |= 0x24; break;
        case Mode2: cw |= 0x40; break;
    }

    /* setup the port direction */
    cw = cw | (pConfig->portDir[0] << 4) | (pConfig->portDir[1] << 1) | pConfig->portDir[2] | (pConfig->portDir[3] << 3);

    /* write the memory mapped register */
    __asm {
        push AX
        push DX

        mov  DX, kParCWAddr
        mov  AL, cw
        out  DX, AL

        pop  DX
        pop  AX
    }
    pConfig->hasInit = 1;

    /* Settup the global parallel configuration */
    gParallelConf = *pConfig;
}


static int  z3ParallelWrite(ParallelPort port, const unsigned char *buffer, const int size)
{
    unsigned char ahVar = 0;
    int i = 0;
    /* check to see if the port direction is ok */
    if (!gParallelConf.hasInit || (gParallelConf.portDir[port] == input))
        return 0;

    /* set the corect AH for Monitor's API */
    switch (port) {
        case A:     ahVar = 1; break;
        case B:     ahVar = 2; break;
        case Clow:
        case Chigh: ahVar = 3; break;
    }

    /* send the data buffer */
    for (; i < size; i++) {
        unsigned char b = buffer[i];
        __asm {
            push AX
            mov  AH, ahVar   ; Settup the parallel port to use
            mov  AL, b       ; Load the data to send
            int  IIOPARALLEL ; Call the Monitor Interrupt to send data
            pop  AX
        }
        /* wait 100ms */
        z3Wait(100);
    }

    /* return the number of bytes written */
    return i;
}


static int  z3ParallelRead(ParallelPort port, unsigned char *buffer, const int size)
{
    unsigned char ahVar = 0;
    int i = 0;
    /* check to see if the port direction is ok */
    if (!gParallelConf.hasInit || (gParallelConf.portDir[port] == output))
        return 0;

    /* set the corect AH for Monitor's API */
    switch (port) {
        case A:     ahVar = 4; break;
        case B:     ahVar = 5; break;
        case Clow:
        case Chigh: ahVar = 6; break;
    }

    /* send the data buffer */
    for (; i < size; i++) {
        __asm {
            push AX
            push SI

            xor  AX, AX
            mov  AH, ahVar               ; Setup the parallel port to use
            int  IIOPARALLEL             ; Call the Monitor interrupt to read data
            mov  SI, buffer              ; Load the buffer address
            add  SI, i                   ; Add the current buffer position
            mov  byte ptr [SI], AL       ; Store the read byte into the buffer

            pop  SI
            pop  AX
        }
        /* wait 100ms */
        z3Wait(100);
    }

    /* return the number of bytes read */
    return i;
}


static void z3SerialInit(const pSerialConfig pConfig)
{
    unsigned char vAL = 0;

    /* set the required bits to configure the serial interface with the Monitor interrupt */
    vAL = (pConfig->baudRate << 5) | (pConfig->parity << 3) | (pConfig->stopBits << 2) | pConfig->wordLength;

    __asm {
        push AX
        xor  AX, AX  ; Initialization of communication port
        mov  AL, vAL ; Parameters of the initialization
        int  IIOSERIAL
        pop  AX
    }
    pConfig->hasInit = 1;

     /* Settup the global serial configuration */
    gSerialConf = *pConfig;
}


static int  z3SerialWrite(SerialPort port, const unsigned char *buffer, const int size)
{
    unsigned char flag = 0;
    int i = 0;
    /* we must make sure we initialized */
    if (!gSerialConf.hasInit)
        return 0;

    /* first activate DTR and wait for DSR */
    __asm {
        push AX
        mov  AH, 0x4            ; Control of the Modem
        mov  AL, 0x1            ; Set DTR active
        int  IIOSERIAL
        pop  AX
    }

    /* wait for DSR */
    while (!flag) {
        __asm {
            push AX
            mov  AH, 0x3        ; Reading of the state
            int  IIOSERIAL
            mov  flag, AH
            pop  AX
        }
        flag = flag & 0x20; /* check is DSR was set */
    }

    for (; i < size; i++) {
        unsigned char b = buffer[i];
        /* check CTS so we can send a byte */
        flag = 0;
        while (!flag) {
            __asm {
                push AX
                mov  AH, 0x3    ; Reading of the state
                int  IIOSERIAL
                mov  flag, AH
                pop  AX
            }
            flag = flag & 0x10; /* check is CTS was set */
        }
        /* finally send a byte */
        __asm {
            push AX
            mov  AH, 0x1        ; Transmission of a character
            mov  AL, b          ; Byte to transmit
            int  IIOSERIAL
            pop  AX
        }
    }

    /* now we should reset DTR */
    __asm {
        push AX
        mov  AH, 0x4            ; Control of the Modem
        mov  AL, 0x0            ; Reset DTR
        int  IIOSERIAL
        pop  AX
    }

    /* return the number of bytes written */
    return i;
}


static int  z3SerialRead(SerialPort port, unsigned char *buffer, const int size)
{
    unsigned char flag = 0;
    int i = 0;
    /* we must make sure we initialized */
    if (!gSerialConf.hasInit)
        return 0;
        
    /* first activate DTR and wait for DSR */
    __asm {
        push AX
        mov  AH, 0x4               ; Control of the Modem
        mov  AL, 0x1               ; Set DTR active
        int  IIOSERIAL
        pop  AX
    }
        
    /* wait for DSR */
    while (!flag) {
        __asm {
            push AX
            mov  AH, 0x3           ; Reading of the state
            int  IIOSERIAL
            mov  flag, AH
            pop  AX
        }
        flag = flag & 0x20; /* check is DSR was set */
    }

    for (; i < size; i++) {
        /* read one byte */
        __asm {
            push AX
            push SI

            mov  AH, 0x4
            mov  AL, 0x3           ; Set DTR and RTS active
            int  IIOSERIAL

            mov  AH, 0x2           ; Reception of a character
            int  IIOSERIAL
            mov  SI, buffer        ; Load the buffer address
            add  SI, i             ; Add the current buffer position
            mov  byte ptr [SI], AL ; Store the read byte into the buffer

            mov  AH, 0x4
            mov  AL, 0x1           ; Set DTR active, RTS inactive
            int  IIOSERIAL

            pop  SI
            pop  AX
        }
    }

   /* now we should reset DTR */
    __asm {
        push AX
        mov  AH, 0x4 ; Control of the Modem
        mov  AL, 0x0 ; Reset DTR
        int  IIOSERIAL
        pop  AX
    }

    /* return the number of bytes read */
    return i;
}


static void z3ADconvert(pConvertBuffer convertBuff)
{
    unsigned char *pBuff = 0;
    int size = 0;
    int i = 0;
 
    if (!convertBuff->size)
        return;

    pBuff = convertBuff->buffer;
    size  = (convertBuff->size < BUFFER_SIZE)? convertBuff->size : BUFFER_SIZE;

    for (; i < size; i++) {
        /* get one byte at a time from the AD access time 135uS */
        __asm {
            push AX
            push SI

            int  IADCONVERT        ; Call to convert the analog input to AL
            mov  SI, pBuff         ; Load the buffer address
            add  SI, i             ; Add the current buffer position
            mov  byte ptr [SI], AL ; Store the read byte into the buffer

            pop  SI
            pop  AX
        }
    }
    /* finally set the received converted data size */
    convertBuff->size = i;
}


static void z3DAconvert(pConvertBuffer convertBuff)
{
    unsigned char *pBuff = 0;
    int size = 0;
    int i = 0;

    if (!convertBuff->size)
        return;

    pBuff = convertBuff->buffer;
    size  = (convertBuff->size < BUFFER_SIZE)? convertBuff->size : BUFFER_SIZE;

    for (; i < size; i++) {
        /* set one byte at a time to the DA access time 100nS */
        __asm {
            push AX
            push SI

            mov  SI, pBuff         ; Load the buffer address
            add  SI, i             ; Add the current buffer position
            mov  AL, byte ptr [SI] ; Get the byte from the buffer
            int  IDACONVERT        ; Call to convert AL to a analog output

            mov  AX, 1
            int  IWAIT             ; Wait 1 milisecond as the convert time is very fast - 100 nS

            pop  SI
            pop  AX
        }
    }
    /* finally set the remaining bytes to send */
    convertBuff->size = convertBuff->size - i;
}

